/***
 * @Description: 生产者管理类 内含3个不同主题发布者
 * @Author: jizhou
 * @Date: 2023-07-13 19:13:44
 * @LastEditors: jizhou
 * @LastEditTime: 2023-07-16 23:57:01
 * @FilePath: /EnvironmentAccess/src/manager/IPCPublisherManager.h
 */
#ifndef FAST_DDS_GENERATED__IPCPublisherManager
#define FAST_DDS_GENERATED__IPCPublisherManager

#include <string>

#include "BaseManager.h"
#include "fastdds/IPCPublisher.h"

using namespace std;

class IPCPublisherManager : public BaseManager<IPCPublisherManager> {
private:
    /***
     * @brief: 构造函数
     * @return {*}
     */
    IPCPublisherManager() = default;


    std::map<std::string, std::shared_ptr<IPCPublisherBase>> publishers_;
    
public:
    /***
     * @brief: 析构函数
     * @return {*}
     */
    
    ~IPCPublisherManager() = default;

    friend class BaseManager;

     template<typename MessageT>
    std::shared_ptr<IPCPublisher<MessageT>> createPublisher(const std::string& topic, size_t qos_profile_depth) {
        auto publisher = std::make_shared<IPCPublisher<MessageT>>(0, topic);
        publishers_[topic] = publisher;
        return publisher;
    }

    template<typename MessageT>
    std::shared_ptr<IPCPublisher<MessageT>> getPublisherByTopic(const std::string& topic) {
        auto it = publishers_.find(topic);
        if (it != publishers_.end()) {
            return std::dynamic_pointer_cast<IPCPublisher<MessageT>>(it->second);
        }
        return nullptr;
    }

    //publish("topic",<T>xxxxx)
    
    void stopAll() {
        auto iter = publishers_.begin();
        for(; iter!= publishers_.end(); iter++) {
            iter->second->stop();
        }
    }

};

#endif